<?php
/*
Plugin Name: CommentXpert
Description: Adds an option for users to mark comments as private, viewable only by admins and the comment author.
Version: 1.1.4
Author: Raghav Chudasama
Author URI: https://profiles.wordpress.org/raghavchudasama/
Text Domain: commentxpert
Domain Path: /languages
License: GPLv3 or later
License URI: https://www.gnu.org/licenses/gpl-3.0.html
*/

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit; // Prevent direct access
}

// Register the admin menu
function cmntxpt_add_admin_menu() {
    add_menu_page(
        __('CommentXpert Settings', 'commentxpert'),
        __('CommentXpert', 'commentxpert'),
        'manage_options',
        'commentxpert-settings',
        'cmntxpt_settings_page',
        'dashicons-admin-comments',
        80
    );

    add_submenu_page( 'commentxpert-settings', __( 'Cleanup Comments', 'commentxpert' ), __( 'Cleanup Comments', 'commentxpert' ), 'manage_options', 'cmntxpt-cleanup-comments', 'cmntxpt_cleanup_comments_page' );
}
add_action('admin_menu', 'cmntxpt_add_admin_menu');

// Activation hook
function cmntxpt_activate() {
    if (function_exists('activate_plugin')) {
        add_option('cmntxpt_redirect', true);
    }
}
register_activation_hook(__FILE__, 'cmntxpt_activate');

// Load the plugin's translated strings
function cmntxpt_load_textdomain() {
    load_plugin_textdomain('commentxpert', false, dirname(plugin_basename(__FILE__)) . '/languages');
}
add_action('plugins_loaded', 'cmntxpt_load_textdomain');

// Deactivation hook
function cmntxpt_deactivate() {
    update_option('cmntxpt_enable_private_comments', 0);
    update_option('cmntxpt_eliminate_website_input', 0);
    update_option('cmntxpt_eliminate_cmntauthor_url', 0);
    update_option('cmntxpt_prevent_url_hyperlinking', 0);
    update_option('cmntxpt_disable_comments_globally', 0);
	update_option('cmntxpt_enable_comment_votes', 0);
	update_option('cmntxpt_enable_comment_captcha', 0);	
}
register_deactivation_hook(__FILE__, 'cmntxpt_deactivate');

// Uninstallation hook - remove the option
function cmntxpt_uninstall() {
    delete_option('cmntxpt_enable_private_comments');
    delete_option('cmntxpt_eliminate_website_input');
    delete_option('cmntxpt_eliminate_cmntauthor_url');
    delete_option('cmntxpt_prevent_url_hyperlinking');
    delete_option('cmntxpt_disable_comments_globally');
	delete_option('cmntxpt_enable_comment_votes');
	delete_option('cmntxpt_enable_comment_captcha');
	delete_option('cmntxpt_enable_rich_text_editor');
}
register_uninstall_hook(__FILE__, 'cmntxpt_uninstall');

// Add "Settings" link next to the "Deactivate" link in the plugin list
function cmntxpt_add_settings_link_to_plugin($links) {
    // Check if the current plugin is the one you want to add the settings link to
    $settings_link = '<a href="' . esc_url(admin_url('/admin.php?page=commentxpert-settings')) . '">' . __('Settings', 'commentxpert') . '</a>';
    
    $links['cmntxpt-settings-link'] = $settings_link;
    
    return $links;
}
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'cmntxpt_add_settings_link_to_plugin');

// Register the settings with sanitization
function cmntxpt_register_settings() {
    register_setting(
        'cmntxpt_settings',
        'cmntxpt_enable_private_comments',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
    register_setting(
        'cmntxpt_settings',
        'cmntxpt_eliminate_website_input',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
    register_setting(
        'cmntxpt_settings',
        'cmntxpt_eliminate_cmntauthor_url',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
    register_setting(
        'cmntxpt_settings',
        'cmntxpt_prevent_url_hyperlinking',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
    register_setting(
        'cmntxpt_settings',
        'cmntxpt_disable_comments_globally',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
	register_setting(
        'cmntxpt_settings',
        'cmntxpt_enable_comment_votes',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
	register_setting(
        'cmntxpt_settings',
        'cmntxpt_enable_comment_captcha',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
	register_setting(
        'cmntxpt_settings',
        'cmntxpt_enable_rich_text_editor',
        array(
            'type'              => 'boolean',
            'sanitize_callback' => 'cmntxpt_sanitize_checkbox', // Sanitization callback
            'default'           => 0,
        )
    );
    
    if (get_option('cmntxpt_redirect', false)) {
		delete_option('cmntxpt_redirect', false);
		wp_safe_redirect( esc_url_raw( admin_url('admin.php?page=commentxpert-settings') ) );
		exit;
	}
}
add_action('admin_init', 'cmntxpt_register_settings');

// Sanitization callback for the checkbox
function cmntxpt_sanitize_checkbox($input) {
    return (int) (bool) $input; // Ensure the value is 0 or 1
}

// Sanitization callback for text input
function cmntxpt_sanitize_text($input) {
    return sanitize_text_field($input); // Securely sanitize the input string
}

// Sanitization callback for number input
function cmntxpt_sanitize_number($input) {
    return is_numeric($input) ? floatval($input) : 0; // Ensure it's a valid number, default to 0 if not
}

// Display the settings page
function cmntxpt_settings_page() {
    ?>
    <style>
        .cmntxpt-toggle {
            position: relative;
            display: inline-block;
            width: 48px;
            height: 24px;
        }
        .cmntxpt-toggle input {
            opacity: 0;
            width: 0;
            height: 0;
        }
        .cmntxpt-toggle .slider {
            position: absolute;
            top: 0;
            left: 0;
            right: 0;
            bottom: 0;
            background-color: #ccc;
            border-radius: 24px;
            transition: 0.3s;
            cursor: pointer;
        }
        .cmntxpt-toggle .slider::before {
            content: "";
            position: absolute;
            height: 18px;
            width: 18px;
            left: 3px;
            bottom: 3px;
            background-color: #fff;
            border-radius: 50%;
            transition: 0.3s;
        }
        .cmntxpt-toggle input:checked + .slider {
            background-color: #007cba;
        }
        .cmntxpt-toggle input:checked + .slider::before {
            transform: translateX(24px);
        }        
        .latest_feature_header::after{
            content: "New";
            color: #fff;
            background-color: red;
            border-radius: 25px;
            font-weight: 800;
            padding: 1px 4px;
            margin-left: 6px;
            margin-top: 0px;
            display: inline-block;
        }
    </style>
    <div class="wrap">
        <h1><?php esc_html_e('CommentXpert Settings', 'commentxpert'); ?></h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('cmntxpt_settings');
            do_settings_sections('cmntxpt_settings');
            ?>
            <table class="form-table">
                <tr valign="top" id="cmntxpt-enable-private-comments">
                    <th scope="row"><?php esc_html_e('Enable Private Comments and Private Replies', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_enable_private_comments" id="cmntxpt_enable_private_comments" value="1" <?php checked(1, get_option('cmntxpt_enable_private_comments', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <input type="hidden" name="private_comment_nonce" value="<?php echo wp_kses_post(wp_create_nonce('private_comment_action')); ?>">
                        <label for="cmntxpt_enable_private_comments"><?php esc_html_e('Allow site users to mark comments as private', 'commentxpert'); ?></label>
                    </td>
                </tr>
                <tr valign="top" id="cmntxpt-eliminate-comment-author-link">
                    <th scope="row"><?php esc_html_e('Eliminate Comment Author Hyper-link', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_eliminate_cmntauthor_url" id="cmntxpt_eliminate_cmntauthor_url" value="1" <?php checked(1, get_option('cmntxpt_eliminate_cmntauthor_url', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <label for="cmntxpt_eliminate_cmntauthor_url"><?php esc_html_e('Eliminate any links or urls from comment author', 'commentxpert'); ?></label>
                    </td>
                </tr>
                <tr valign="top" id="cmntxpt-eliminate-comment-website-input">
                    <th scope="row"><?php esc_html_e('Eliminate Website Input from Comment Form', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_eliminate_website_input" id="cmntxpt_eliminate_website_input" value="1" <?php checked(1, get_option('cmntxpt_eliminate_website_input', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <label for="cmntxpt_eliminate_website_input"><?php esc_html_e('Eliminate website input from comment form for non-logged in users', 'commentxpert'); ?></label>
                    </td>
                </tr>
                <tr valign="top" id="cmntxpt-prevent-comment-auto-hyperlink">
                    <th scope="row"><?php esc_html_e('Prevent automatic hyperlinking of URLs in comments', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_prevent_url_hyperlinking" id="cmntxpt_prevent_url_hyperlinking" value="1" <?php checked(1, get_option('cmntxpt_prevent_url_hyperlinking', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <label for="cmntxpt_prevent_url_hyperlinking"><?php esc_html_e('Turn off the conversion of URLs to clickable links in comments', 'commentxpert'); ?></label>
                    </td>
                </tr>
                <tr valign="top" id="cmntxpt-turn-off-comments-sitewide">
                    <th scope="row"><?php esc_html_e('Turn Off Comments Across the Entire Site', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_disable_comments_globally" id="cmntxpt_disable_comments_globally" value="1" <?php checked(1, get_option('cmntxpt_disable_comments_globally', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <label for="cmntxpt_disable_comments_globally"><?php esc_html_e('Disable comments globally on entire website', 'commentxpert'); ?></label>
                    </td>
                </tr>
                <tr valign="top" id="cmntxpt-enable-rich-text-editor">
					<th class="latest_feature_header" scope="row"><?php esc_html_e('Enable Rich Text Editor in Comment Form', 'commentxpert'); ?></th>
					<td>
						<label class="cmntxpt-toggle">
							<input type="checkbox" name="cmntxpt_enable_rich_text_editor" id="cmntxpt_enable_rich_text_editor" value="1" <?php checked(1, get_option('cmntxpt_enable_rich_text_editor', 0)); ?> />
							<span class="slider"></span>
						</label>
						<label for="cmntxpt_enable_rich_text_editor"><?php esc_html_e('Enable TinyMCE rich text editor in the comment message field', 'commentxpert'); ?></label>
					</td>
				</tr>
				<tr valign="top" id="cmntxpt-enable-comment-votes">
                    <th class="latest_feature_header" scope="row"><?php esc_html_e('Enable Comment Upvote/Downvote', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_enable_comment_votes" id="cmntxpt_enable_comment_votes" value="1" <?php checked(1, get_option('cmntxpt_enable_comment_votes', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <label for="cmntxpt_enable_comment_votes"><?php esc_html_e('Allow site users to upvote/downvote each comment', 'commentxpert'); ?></label>
                    </td>
                </tr>
				<tr valign="top" id="cmntxpt-enable-comment-captcha">
                    <th class="latest_feature_header" scope="row"><?php esc_html_e('Enable CAPTCHA in Comment Form', 'commentxpert'); ?></th>
                    <td>
                        <label class="cmntxpt-toggle">
                            <input type="checkbox" name="cmntxpt_enable_comment_captcha" id="cmntxpt_enable_comment_captcha" value="1" <?php checked(1, get_option('cmntxpt_enable_comment_captcha', 0)); ?> />
                            <span class="slider"></span>
                        </label>
                        <label for="cmntxpt_enable_comment_captcha"><?php esc_html_e('Adds a CAPTCHA in comment form to prevent spam', 'commentxpert'); ?></label>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

// Check if private comments are enabled
function cmntxpt_is_private_comments_enabled() {
    return get_option('cmntxpt_enable_private_comments', 0) == 1;
}

// Check if remove website input from comment form is enabled
function cmntxpt_is_eliminate_website_input_enabled() {
    return get_option('cmntxpt_eliminate_website_input', 0) == 1;
}

// Check if remove url from comment author is enabled
function cmntxpt_is_eliminate_cmntauthor_url_enabled() {
    return get_option('cmntxpt_eliminate_cmntauthor_url', 0) == 1;
}

// Check if prevent comments urls hyperlinking is enabled
function cmntxpt_is_prevent_url_hyperlinking_enabled() {
    return get_option('cmntxpt_prevent_url_hyperlinking', 0) == 1;
}

// Check if comments are disabled globally on the site
function cmntxpt_is_disable_comments_globally() {
    return get_option('cmntxpt_disable_comments_globally', 0) == 1;
}

// Check if comment votes are enabled on the site
function cmntxpt_is_comment_votes_enabled() {
    return get_option('cmntxpt_enable_comment_votes', 0) == 1;
}

// Check if comment captcha is enabled on the site
function cmntxpt_is_comment_captcha_enabled() {
    return get_option('cmntxpt_enable_comment_captcha', 0) == 1;
}

// Check if comment rich text editor field is enabled.
function cmntxpt_is_rich_text_editor_enabled() {
    return get_option('cmntxpt_enable_rich_text_editor', 0) == 1;
}

// Check if comment registration is enabled
function cmntxpt_check_comment_registration() {
    return get_option('comment_registration') == 1;
}

// Display admin notice if comment registration is not enabled
function cmntxpt_check_comment_registration_notice() {
    if (cmntxpt_is_private_comments_enabled() && !cmntxpt_check_comment_registration()) {
        $link = esc_url(admin_url('options-discussion.php#comment_registration'));
        echo '<div class="notice notice-warning is-dismissible">';
        // Prepare the translation text with placeholders
        $translation_text = __('To enable private comments, please enable the <strong>"Users must be registered and logged in to comment"</strong> option in the', 'commentxpert');

        // Output the final paragraph
        echo '<p>' . wp_kses_post($translation_text) . wp_kses_post(sprintf('<a href="%s">Discussion Settings</a>', esc_url($link))) . '</p>';
        echo '</div>';
    }
}
add_action('admin_notices', 'cmntxpt_check_comment_registration_notice');

// Add the "Private Comment" checkbox above the submit button
if (cmntxpt_is_private_comments_enabled()) {

    if(cmntxpt_check_comment_registration()){
        // Add the "Private Comment" checkbox just above the Post Comment button
        function cmntxpt_private_cmnt_chkbox_above_submit($submit_field) {
            // Define a filterable label
            $private_comment_label = apply_filters('cmntxpt_private_comment_label', __('Post this comment as private', 'commentxpert'));

            // Create the checkbox HTML
            $private_comment_checkbox = '
                <p class="comment-form-private">
                    <input id="private_comment" name="private_comment" type="checkbox" />
                    <label for="private_comment">' . esc_html($private_comment_label) . '</label>
                    <input type="hidden" name="post_private_comment_nonce" value="'. wp_kses_post(wp_create_nonce('post_private_comment_action')) .'">
                </p>';

            // Return the updated submit field with the checkbox added
            return $private_comment_checkbox . $submit_field;
        }
        add_filter('comment_form_submit_field', 'cmntxpt_private_cmnt_chkbox_above_submit');

        // Save the private comment metadata
        function cmntxpt_save_private_comment_meta($comment_id) {
            if (isset($_POST['private_comment'])) {
                // Check if nonce is set before using it
                if (isset($_POST['post_private_comment_nonce'])) {
                    // Sanitize and unslash the nonce
                    $nonce = sanitize_text_field(wp_unslash($_POST['post_private_comment_nonce']));

                    // Verify the nonce
                    if (wp_verify_nonce($nonce, 'post_private_comment_action')) {
                        add_comment_meta($comment_id, 'private_comment', true);
                    } else {
                        // Nonce verification failed, handle the error
                        // error_log('Nonce verification failed for private comment.');
                    }
                } else {
                    // Nonce not set, handle the error
                    // error_log('Nonce is missing for private comment.');
                }
            }
        }
        add_action('comment_post', 'cmntxpt_save_private_comment_meta');

        // Filter comments to exclude private comments for non-admins and non-authors
        function cmntxpt_filter_private_comments($comments) {
            $current_user = wp_get_current_user();
            foreach ($comments as $key => $comment) {
                $private_comment = get_comment_meta($comment->comment_ID, 'private_comment', true);
                if ($private_comment && !current_user_can('administrator') && $comment->user_id != $current_user->ID) {
                    unset($comments[$key]);
                }
            }
            return $comments;
        }
        add_filter('the_comments', 'cmntxpt_filter_private_comments');

        // Modify the admin comments list to show private comments separately
        function cmntxpt_display_private_comments_in_admin($comments) {
            $private_comments = array_filter($comments, function($comment) {
                return get_comment_meta($comment->comment_ID, 'private_comment', true);
            });
            if ($private_comments) {
                echo '<h2>' . wp_kses_post(__('Private Comments Awaiting Approval', 'commentxpert')) . '</h2><ul>';
                foreach ($private_comments as $private_comment) {
                    echo '<li>' . wp_kses_post(get_comment_text($private_comment->comment_ID)) . ' - <a href="' . esc_url(admin_url('comment.php?action=approve&c=' . $private_comment->comment_ID)) . '">' . wp_kses_post(__('Approve', 'commentxpert')) . '</a></li>';
                }
                echo '</ul>';
            }
        }
        add_action('edit_comment', 'cmntxpt_display_private_comments_in_admin');

        // Approve comment with the option to make it public
		function cmntxpt_approve_private_comment($comment_id) {
			if (isset($_GET['approve']) && $_GET['approve'] === '1' && isset($_REQUEST['_wpnonce'])) {
				// Unslash and sanitize the nonce, then verify it
				$nonce = sanitize_text_field(wp_unslash($_REQUEST['_wpnonce']));
				if (!empty(sanitize_text_field( wp_unslash( $_GET['c']) ) ) && wp_verify_nonce(sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'approve-comment_' . sanitize_text_field( wp_unslash( $_GET['c']) ))) {
					// Approve the comment
					wp_set_comment_status($comment_id, 'approve');

					// Check if the comment should be made public
					if (isset($_POST['make_public'])) {
						delete_comment_meta($comment_id, 'private_comment');
					}
				} else {
					// Invalid nonce - handle the error (optional)
					wp_redirect(admin_url('edit-comments.php?nonce_failed=true'));
                    exit;
				}
			}
		}
		add_action('comment_post', 'cmntxpt_approve_private_comment');

        // Add approval options in the comment edit screen
        function cmntxpt_add_approval_options($comment) {
            if (get_comment_meta($comment->comment_ID, 'private_comment', true)) {
                echo '<p><label><input type="checkbox" name="make_public" value="1" /> ' . wp_kses_post(__('Make this comment public', 'commentxpert')) . '</label></p>';
            }
        }
        add_action('comment_edit_form', 'cmntxpt_add_approval_options');

        // Add a custom action button to make private comment public
        function cmntxpt_add_make_public_comment_action($actions, $comment) {
            if (get_comment_meta($comment->comment_ID, 'private_comment', true)) {
                $url = wp_nonce_url(admin_url('comment.php?action=make_public_comment&c=' . $comment->comment_ID), 'make_public_comment_' . $comment->comment_ID);
                $actions['make_public'] = '<a href="' . esc_url($url) . '">' . __('Make Public', 'commentxpert') . '</a>';
            } else {
                $url = wp_nonce_url(admin_url('comment.php?action=make_private_comment&c=' . $comment->comment_ID), 'make_private_comment_' . $comment->comment_ID);
                $actions['make_private'] = '<a href="' . esc_url($url) . '">' . __('Make Private', 'commentxpert') . '</a>';
            }
            return $actions;
        }
        add_filter('comment_row_actions', 'cmntxpt_add_make_public_comment_action', 10, 2);

        // Handle making the private comment public or vice versa
		function cmntxpt_handle_make_public_comment_action() {
			// Check if the action is 'make_public_comment' or 'make_private_comment'
			if (isset($_GET['action'])) {
				$comment_id = isset($_GET['c']) ? absint(sanitize_text_field( wp_unslash( $_GET['c']) ) ) : 0;

				// Verify the nonce based on the action and comment ID
				$nonce_action = '';
				if ($_GET['action'] == 'make_public_comment') {
					$nonce_action = 'make_public_comment_' . $comment_id;
				} elseif ($_GET['action'] == 'make_private_comment') {
					$nonce_action = 'make_private_comment_' . $comment_id;
				}

				// Ensure the nonce is valid
				check_admin_referer($nonce_action);

                // Generate and pass nonce for the success message
				$redirect_nonce = wp_create_nonce('comment_status_change_nonce');

				if (current_user_can('edit_comment', $comment_id)) {
					if ($_GET['action'] == 'make_public_comment') {
						// Remove 'private_comment' meta to make the comment public
						delete_comment_meta($comment_id, 'private_comment');

                        $message = 1;
                        wp_redirect(add_query_arg(['message' => $message, '_wpnonce' => $redirect_nonce], admin_url('edit-comments.php')));
						exit;

					} elseif ($_GET['action'] == 'make_private_comment') {
						// Add 'private_comment' meta to make the comment private
						add_comment_meta($comment_id, 'private_comment', true);

                        $message = 2;
                        wp_redirect(add_query_arg(['message' => $message, '_wpnonce' => $redirect_nonce], admin_url('edit-comments.php')));
						exit;
					}
				}
			}
		}
		add_action('admin_init', 'cmntxpt_handle_make_public_comment_action');

        // Display a success notice after making the comment public or private
		function cmntxpt_show_make_public_comment_notice() {
			if (isset($_GET['message']) && isset($_REQUEST['_wpnonce'])) {
				// Verify and sanitize the nonce
				$nonce = sanitize_text_field(wp_unslash($_REQUEST['_wpnonce']));
				if (wp_verify_nonce($nonce, 'comment_status_change_nonce')) {
                    $message = sanitize_text_field(wp_unslash($_GET['message']));
					if ($message == '1') {
						echo '<div class="notice notice-success is-dismissible"><p>' . wp_kses_post(__('Comment made public successfully.', 'commentxpert')) . '</p></div>';
					} elseif ($message == '2') {
						echo '<div class="notice notice-success is-dismissible"><p>' . wp_kses_post(__('Comment made private successfully.', 'commentxpert')) . '</p></div>';
					}
				}
			}
		}
		add_action('admin_notices', 'cmntxpt_show_make_public_comment_notice');

        // Register custom bulk actions
        function cmntxpt_register_bulk_actions($bulk_actions) {
            $bulk_actions['make_public_comments'] = __('Make Public', 'commentxpert');
            $bulk_actions['make_private_comments'] = __('Make Private', 'commentxpert');
            return $bulk_actions;
        }
        add_filter('bulk_actions-edit-comments', 'cmntxpt_register_bulk_actions');

        // Handle custom bulk actions
        function cmntxpt_handle_bulk_actions($redirect_to, $action, $comment_ids) {
            if ($action === 'make_public_comments') {
                foreach ($comment_ids as $comment_id) {
                    // Remove 'private_comment' meta to make it public
                    delete_comment_meta($comment_id, 'private_comment');
                }
                $redirect_to = add_query_arg('bulk_make_public', count($comment_ids), $redirect_to);
            }

            if ($action === 'make_private_comments') {
                foreach ($comment_ids as $comment_id) {
                    // Add 'private_comment' meta to make it private
                    update_comment_meta($comment_id, 'private_comment', true);
                }
                $redirect_to = add_query_arg('bulk_make_private', count($comment_ids), $redirect_to);
            }

            return $redirect_to;
        }
        add_filter('handle_bulk_actions-edit-comments', 'cmntxpt_handle_bulk_actions', 10, 3);

        // Display admin notices for bulk actions
        function cmntxpt_bulk_action_notices() {
            if ( ! empty( $_REQUEST['bulk_make_public'] ) && check_admin_referer('bulk-comments') ) {
                $count = intval( $_REQUEST['bulk_make_public'] );

                /* translators: %s is the number of comments made public. */
                $message = sprintf(_n( '%s comment made public successfully.', '%s comments made public successfully.', $count, 'commentxpert' ),number_format_i18n( $count ));

                printf('<div id="message" class="updated notice notice-success is-dismissible"><p>%s</p></div>', esc_html( $message ));
            }

            if ( ! empty( $_REQUEST['bulk_make_private'] ) && check_admin_referer('bulk-comments')  ) {
                $count = intval( $_REQUEST['bulk_make_private'] );

                /* translators: %s is the number of comments made private. */
                $message = sprintf(_n( '%s comment made private successfully.', '%s comments made private successfully.', $count, 'commentxpert' ), number_format_i18n( $count ));

                printf('<div id="message" class="updated notice notice-success is-dismissible"><p>%s</p></div>', esc_html( $message ));
            }
        }
        add_action( 'admin_notices', 'cmntxpt_bulk_action_notices' );

        // Conditional comments count based on user permissions and including replies to private comments for authors/admins
        function cmntxpt_conditional_comment_count($count) {
            if (is_single()) { // Check if it's a single post
                global $post;
                $comments = get_comments(['post_id' => $post->ID, 'status' => 'approve']); // Get only approved comments

                $visible_count = 0;
                $current_user = wp_get_current_user();

                // Collect private comment IDs and their authors
                $private_comment_ids = [];
                $private_comment_authors = [];

                foreach ($comments as $comment) {
                    $private_comment = get_comment_meta($comment->comment_ID, 'private_comment', true);
                    if ($private_comment) {
                        $private_comment_ids[] = $comment->comment_ID; // Store the ID of private comments
                        $private_comment_authors[] = $comment->user_id; // Store the author ID of private comments
                    }
                }

                // Count comments based on user permissions
                foreach ($comments as $comment) {
                    $is_reply = $comment->comment_parent > 0; // Check if it's a reply

                    if (!$is_reply) {
                        // Count the top-level comment if it's public
                        if (!$private_comment || current_user_can('administrator') || in_array($current_user->ID, $private_comment_authors)) {
                            $visible_count++;
                        }
                    } else {
                        // If it's a reply, check if its parent is a private comment
                        if (in_array($comment->comment_parent, $private_comment_ids)) {
                            // Allow reply display to comment author or admin only
                            if (in_array($current_user->ID, $private_comment_authors) || current_user_can('administrator')) {
                                $visible_count++;
                            }
                        } else {
                            $visible_count++;
                        }
                    }
                }
                return $visible_count;
            }

            return $count;
        }
        add_filter('get_comments_number', 'cmntxpt_conditional_comment_count');
    }
}

// Remove website input from comment form if option enabled
if (cmntxpt_is_eliminate_website_input_enabled()) {

    function cmntxpt_eliminate_website_input($fields) {
        if (isset($fields['url'])) {
            unset($fields['url']);
        }
        return $fields;
    }
    add_filter('comment_form_default_fields', 'cmntxpt_eliminate_website_input');

}

if (cmntxpt_is_eliminate_cmntauthor_url_enabled()) {
    
    // Remove hyperlink from the comment author's name and URL
    function cmntxpt_eliminate_comment_author_hyperlink($author_link, $author_id) {
        // Get the comment author's name and return it without a link
        $author_name = get_comment_author($author_id);
        return $author_name; // Return just the author's name without any hyperlink
    }
    add_filter('get_comment_author_link', 'cmntxpt_eliminate_comment_author_hyperlink', 10, 2);

    // Remove the URL (and the link) from the comment author's URL
    function cmntxpt_eliminate_comment_author_url($url, $comment_id) {
        return ''; // Return an empty string to remove the URL completely
    }
    add_filter('get_comment_author_url', 'cmntxpt_eliminate_comment_author_url', 10, 2);

}

if (cmntxpt_is_prevent_url_hyperlinking_enabled()) {

    // Disable turning URLs into hyperlinks in comments
    function cmntxpt_disable_hyperlining_in_comments($comment_content) {
        // Strip any URLs from the comment content to prevent auto-linking
        $comment_content = preg_replace('#http[s]?://[^\s]+#', '', $comment_content);
        return $comment_content;
    }
    add_filter('pre_comment_content', 'cmntxpt_disable_hyperlining_in_comments');

}

if (cmntxpt_is_disable_comments_globally()) {

    // Close comments on the front-end
    function cmntxpt_turn_off_comments() {
        return false;
    }
    add_filter('comments_open', 'cmntxpt_turn_off_comments', 15, 2);
    add_filter('pings_open', 'cmntxpt_turn_off_comments', 15, 2);

}

if (cmntxpt_is_comment_votes_enabled()) {

    // Add upvote/downvote buttons to comments
	function cmntxpt_add_vote_buttons($comment_text, $comment) {
		if (is_admin()) return $comment_text;

		if (!isset($comment) || !is_object($comment) || !isset($comment->comment_ID)) {
			return $comment_text;
		}

		$upvotes = get_comment_meta($comment->comment_ID, 'cmntxpt_upvotes', true) ?: 0;
		$downvotes = get_comment_meta($comment->comment_ID, 'cmntxpt_downvotes', true) ?: 0;

		ob_start();
		?>
		<div class="cmntxpt-vote-buttons">
			<button class="button button-small cmntxpt-upvote" data-comment-id="<?php echo esc_attr($comment->comment_ID); ?>">
				<span class="dashicons dashicons-thumbs-up"></span> <span class="cmntxpt-upvote-count"><?php echo esc_html($upvotes); ?></span>
			</button>
			<button class="button button-small cmntxpt-downvote" data-comment-id="<?php echo esc_attr($comment->comment_ID); ?>">
				<span class="dashicons dashicons-thumbs-down"></span> <span class="cmntxpt-downvote-count"><?php echo esc_html($downvotes); ?></span>
			</button>
		</div>
		<?php
		$comment_text .= ob_get_clean();
		return $comment_text;
	}
	add_filter('comment_text', 'cmntxpt_add_vote_buttons', 10, 2);

	// Handle AJAX request for upvoting and downvoting
	function cmntxpt_handle_vote() {
		check_ajax_referer('cmntxpt_vote_nonce', 'nonce');

		// Validate if 'comment_id' is set and is a valid number
		if ( ! isset( $_POST['comment_id'] ) || ! is_numeric( $_POST['comment_id'] ) ) {
			wp_send_json_error(['message' => __('Invalid request', 'commentxpert')]);
		}

		$comment_id = intval( $_POST['comment_id'] );

		// Check if 'vote_type' is set and unslash it before sanitization
		if ( ! isset( $_POST['vote_type'] ) || ! in_array( wp_unslash( $_POST['vote_type'] ), ['upvote', 'downvote'] ) ) {
			wp_send_json_error(['message' => __('Invalid request', 'commentxpert')]);
		}

		// Unsure if slashes are added, so we unslash before sanitizing
		$vote_type = sanitize_text_field( wp_unslash( $_POST['vote_type'] ) );

		$meta_key = $vote_type === 'upvote' ? 'cmntxpt_upvotes' : 'cmntxpt_downvotes';
		$votes = get_comment_meta( $comment_id, $meta_key, true ) ?: 0;
		$votes++;

		update_comment_meta( $comment_id, $meta_key, $votes );
		wp_send_json_success(['votes' => $votes]);
	}
	add_action( 'wp_ajax_cmntxpt_handle_vote', 'cmntxpt_handle_vote' );
	add_action( 'wp_ajax_nopriv_cmntxpt_handle_vote', 'cmntxpt_handle_vote' );

	// Enqueue the script
	function cmntxpt_enqueue_vote_script() {
		if (is_singular() && comments_open()) {
			wp_enqueue_script('cmntxpt-vote-script', plugin_dir_url(__FILE__) . 'assets/js/vote.js', ['jquery'], microtime(), true);

			wp_localize_script('cmntxpt-vote-script', 'cmntxptVote', [
				'ajax_url' => admin_url('admin-ajax.php'),
				'nonce' => wp_create_nonce('cmntxpt_vote_nonce')
			]);
		}
	}
	add_action('wp_enqueue_scripts', 'cmntxpt_enqueue_vote_script');

	function cmntxpt_enqueue_styles() {
		wp_enqueue_style('dashicons'); // Load Dashicons
		?>
		<style>
			.cmntxpt-vote-buttons {
				display: flex;
				gap: 8px;
				margin-top: 10px;
			}
			.cmntxpt-vote-buttons .button {
				display: flex;
				align-items: center;
				justify-content: center;
				width: 32px;
				height: 32px;
				padding: 0;
			}
			.cmntxpt-vote-buttons .dashicons {
				font-size: 20px;
				line-height: 1;
			}
		</style>
		<?php
	}
	add_action('wp_head', 'cmntxpt_enqueue_styles');

}

/**
 * Handle the cleanup of all pending/spam comments
 */
function cmntxpt_comment_cleanup_process() {
    global $wpdb;

    // Custom confirmation text
    $confirmation_text = __( 'Cleanup Comments', 'commentxpert' );

    // Verify the security nonce
    if ( ! isset( $_POST['_wpnonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'cmntxpt-cleanup-comments' ) ) {
        wp_die( esc_html__( 'Nonce verification failed', 'commentxpert' ) );
    }

    // Validate user input for confirmation text
    if ( isset( $_POST['cleanup_confirm'] ) && sanitize_text_field( wp_unslash( $_POST['cleanup_confirm'] ) ) === $confirmation_text ) {

        // First, get the count of comments before deletion for accurate messaging
        $comment_ids = wp_cache_get( 'cmntxpt_cleanup_comment_ids' );
		
		$pending_comments = get_comments([
			'status' => 'hold',
			'fields' => 'ids', // Only fetch comment IDs
		]);
        
        if ( false === $comment_ids ) {
            // $comment_ids = $wpdb->get_col( $wpdb->prepare( "SELECT comment_ID FROM {$wpdb->comments} WHERE comment_approved = %d", 0 ) );

			$comment_ids = !empty($pending_comments) ? $pending_comments : [];
			
            wp_cache_set( 'cmntxpt_cleanup_comment_ids', $comment_ids, '', 600 ); // Cache for 10 minutes
        }

        $deleted_count = count( $comment_ids );

        if ( $deleted_count > 0 ) {
            // Delete comments securely using a prepared statement
			if ( ! empty( $pending_comments ) ) {
				foreach ( $pending_comments as $comment_id ) {
					wp_delete_comment( $comment_id, true ); // 'true' forces a permanent delete
				}
			}

			// Cleanup orphaned comment meta
			// phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery
			$orphaned_meta = $wpdb->get_results(
				"SELECT meta_id 
				FROM {$wpdb->commentmeta} 
				LEFT JOIN {$wpdb->comments} 
				ON {$wpdb->commentmeta}.comment_id = {$wpdb->comments}.comment_ID 
				WHERE {$wpdb->comments}.comment_ID IS NULL"
			);

			if ( ! empty( $orphaned_meta ) ) {
				foreach ( $orphaned_meta as $meta ) {
					delete_metadata_by_mid( 'comment', $meta->meta_id );
				}
			}

            // Clear the cache after cleanup
            wp_cache_delete( 'cmntxpt_cleanup_comment_ids' );

            /* translators: Successfully deleted %d pending/spam comments. */
            return sprintf( esc_html__( 'Successfully deleted %d pending/spam comments.', 'commentxpert' ), $deleted_count );
        }

        return esc_html__( 'No pending or spam comments were found.', 'commentxpert' );
    }

    // Return error message if confirmation text is incorrect
    return esc_html__( 'The confirmation text you entered is incorrect. Please try again.', 'commentxpert' );
}

/**
 * Render the settings page for cleanup pending/spam comments
 */
function cmntxpt_cleanup_comments_page() {
    // Check if the user has the necessary permissions
    if ( ! current_user_can( 'manage_options' ) ) {
        return;
    }

    // Process the form submission result
    $feedback_message = '';
    if ( !empty( $_SERVER['REQUEST_METHOD'] ) && $_SERVER['REQUEST_METHOD'] === 'POST' ) {
        $feedback_message = cmntxpt_comment_cleanup_process();
    }

    // Fetch all pending and spam comment IDs
    $cleanup_comments_ids = cmntxpt_get_cleanup_comment_ids();
    $cleanup_comments_count = count( $cleanup_comments_ids );

    // Display the page
    ?>
    <div class="wrap">
        <h2><?php esc_html_e( 'Cleanup Pending/Spam Comments', 'commentxpert' ); ?></h2>

        <?php if ( $feedback_message ): ?>
            <div class="notice <?php echo strpos( $feedback_message, 'successfully' ) !== false ? 'updated' : 'error'; ?>">
                <p><?php echo esc_html( $feedback_message ); ?></p>
            </div>
        <?php endif; ?>

        <?php if ( $cleanup_comments_count > 0 ): ?>
            <p>
                <?php
                    /* translators: %s is the number of pending/spam comments. */
                    $message = sprintf(_n('There is %s pending/spam comment on your site. Do you want to clean it up?', 'There are %s pending/spam comments on your site. Do you want to clean them up?', $cleanup_comments_count, 'commentxpert'), number_format_i18n($cleanup_comments_count));

                    printf( '<div class="notice notice-warning is-dismissible"><p>%s</p></div>', esc_html($message) );
                ?>
            </p>

            <p><?php echo wp_kses_post( __( 'To confirm the pending/spam comments cleanup, please type <b>Cleanup Comments</b>', 'commentxpert' ) ); ?></p>

            <p>
                <strong><?php esc_html_e( 'Important Note: ', 'commentxpert' ); ?><strong>
                <?php esc_html_e( "Once the comments are cleaned up, they cannot be restored.", 'commentxpert' ); ?>
            </p>

            <form action="" method="post">
                <?php wp_nonce_field( 'cmntxpt-cleanup-comments' ); ?>
                <input name="cleanup_confirm" type="text" style="width: 80%;">
                <p class="submit">
                    <input type="submit" class="button-primary" value="<?php esc_html_e( 'Cleanup Pending/Spam Comments', 'commentxpert' ); ?>">
                </p>
            </form>
        <?php else: ?>
            <p><?php esc_html_e( 'Your site has no pending or spam comments.', 'commentxpert' ); ?></p>
        <?php endif; ?>
    </div>
    <?php
}

/**
 * Retrieve all IDs of unapproved comments
 *
 * @return array
 */
function cmntxpt_get_cleanup_comment_ids() : array {
    global $wpdb;

    // Define a cache key
    $cache_key = 'cmntxpt_cleanup_comment_ids';
    $cached_ids = wp_cache_get( $cache_key );

    // Return cached result if available
    if ( false !== $cached_ids ) {
        return $cached_ids;
    }

    // Fetch pending comment IDs using WordPress functions
	$pending_comment_ids = get_comments( array(
		'status'  => 'hold',  // 'hold' means pending comments
		'fields'  => 'ids',   // Only get the comment IDs
	) );

    // Store the result in cache for 10 minutes
    wp_cache_set( $cache_key, $pending_comment_ids, '', 600 );

    return $pending_comment_ids;
}

function cmntxpt_clear_comment_cache() {
    wp_cache_delete( 'cmntxpt_cleanup_comment_ids' );
}
add_action( 'comment_post', 'cmntxpt_clear_comment_cache' );
add_action( 'edit_comment', 'cmntxpt_clear_comment_cache' );
add_action( 'delete_comment', 'cmntxpt_clear_comment_cache' );

// Add a new submenu for CAPTCHA configuration
function cmntxpt_add_captcha_submenu() {
    add_submenu_page(
        'commentxpert-settings',
        __('Captcha Configuration', 'commentxpert'),
        __('Captcha Configuration', 'commentxpert'),
        'manage_options',
        'cmntxpt-captcha-settings',
        'cmntxpt_captcha_settings_page'
    );
}
add_action('admin_menu', 'cmntxpt_add_captcha_submenu');

// Register settings
function cmntxpt_register_captcha_settings() {
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_captcha_type',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
            'default'           => 'v2',
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_recaptcha_site_key',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_recaptcha_secret_key',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_recaptcha_v3_site_key',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_recaptcha_v3_secret_key',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_recaptcha_v3_threshold',
        array(
            'type'              => 'number',
            'sanitize_callback' => 'cmntxpt_sanitize_number',
            'default'           => 0.5,
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_hcaptcha_site_key',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
        )
    );	
	register_setting(
        'cmntxpt-captcha-settings',
        'cmntxpt_hcaptcha_secret_key',
        array(
            'type'              => 'string',
            'sanitize_callback' => 'cmntxpt_sanitize_text',
        )
    );
}
add_action('admin_init', 'cmntxpt_register_captcha_settings');

// Create the reCAPTCHA settings page
function cmntxpt_captcha_settings_page() {
    $recaptcha_type = get_option('cmntxpt_captcha_type', 'v2');
    ?>
    <div class="wrap">
        <h1><?php esc_html_e('Captcha Configuration', 'commentxpert'); ?></h1>
        <form method="post" action="options.php">
            <?php settings_fields('cmntxpt-captcha-settings'); ?>
            <?php do_settings_sections('cmntxpt-captcha-settings'); ?>

            <h2>Select CAPTCHA Type</h2>
            <label>
                <input type="radio" name="cmntxpt_captcha_type" value="v2" <?php checked($recaptcha_type, 'v2'); ?>> reCAPTCHA v2
            </label>
            <label>
                <input type="radio" name="cmntxpt_captcha_type" value="v3" <?php checked($recaptcha_type, 'v3'); ?>> reCAPTCHA v3
            </label>
            <label>
                <input type="radio" name="cmntxpt_captcha_type" value="hcaptcha" <?php checked($recaptcha_type, 'hcaptcha'); ?>> hCaptcha
            </label>

            <div id="recaptcha-v2-settings" style="display: <?php echo $recaptcha_type === 'v2' ? 'block' : 'none'; ?>;">
                <h2>reCAPTCHA v2 Settings</h2>
                <table class="form-table">
                    <tr>
                        <th>Site Key</th>
                        <td>
                            <input type="text" name="cmntxpt_recaptcha_site_key" value="<?php echo esc_attr(get_option('cmntxpt_recaptcha_site_key')); ?>" class="regular-text">
                        </td>
                    </tr>
                    <tr>
                        <th>Secret Key</th>
                        <td>
                            <input type="text" name="cmntxpt_recaptcha_secret_key" value="<?php echo esc_attr(get_option('cmntxpt_recaptcha_secret_key')); ?>" class="regular-text">
                        </td>
                    </tr>
                </table>
            </div>

            <div id="recaptcha-v3-settings" style="display: <?php echo $recaptcha_type === 'v3' ? 'block' : 'none'; ?>;">
                <h2>reCAPTCHA v3 Settings</h2>
                <table class="form-table">
                    <tr>
                        <th>Site Key</th>
                        <td>
                            <input type="text" name="cmntxpt_recaptcha_v3_site_key" value="<?php echo esc_attr(get_option('cmntxpt_recaptcha_v3_site_key')); ?>" class="regular-text">
                        </td>
                    </tr>
                    <tr>
                        <th>Secret Key</th>
                        <td>
                            <input type="text" name="cmntxpt_recaptcha_v3_secret_key" value="<?php echo esc_attr(get_option('cmntxpt_recaptcha_v3_secret_key')); ?>" class="regular-text">
                        </td>
                    </tr>
                    <tr>
                        <th>Score Threshold</th>
                        <td>
                            <input type="number" step="0.1" min="0" max="1" name="cmntxpt_recaptcha_v3_threshold" value="<?php echo esc_attr(get_option('cmntxpt_recaptcha_v3_threshold', '0.5')); ?>" class="regular-text">
                            <p class="description">Threshold should be between 0 and 1, default: 0.5</p>
                        </td>
                    </tr>
                </table>
            </div>

            <div id="hcaptcha-settings" style="display: <?php echo $recaptcha_type === 'hcaptcha' ? 'block' : 'none'; ?>;">
                <h2>hCaptcha Settings</h2>
                <table class="form-table">
                    <tr>
                        <th>Site Key</th>
                        <td>
                            <input type="text" name="cmntxpt_hcaptcha_site_key" value="<?php echo esc_attr(get_option('cmntxpt_hcaptcha_site_key')); ?>" class="regular-text">
                        </td>
                    </tr>
                    <tr>
                        <th>Secret Key</th>
                        <td>
                            <input type="text" name="cmntxpt_hcaptcha_secret_key" value="<?php echo esc_attr(get_option('cmntxpt_hcaptcha_secret_key')); ?>" class="regular-text">
                        </td>
                    </tr>
                </table>
            </div>
            <div>
                <?php
                    printf(
                        '<p><strong>' . esc_html__('Note:', 'commentxpert') . '</strong> ' . wp_kses(
                            /* translators: %s is the link to CommentXpert settings page */
                            esc_html__('If captcha is disabled in %s, it won’t appear and work, even if configured.', 'commentxpert'),
                            array(
                                'a' => array(
                                    'href' => array(),
                                    'target' => array()
                                )
                            )
                        ) . '</p>',
                        '<a href="' . esc_url(admin_url('admin.php?page=commentxpert-settings#cmntxpt-enable-comment-captcha')) . '" target="_blank">' . esc_html__('CommentXpert settings', 'commentxpert') . '</a>'
                    );                
                ?>
            </div>
            <?php submit_button(); ?>
        </form>
    </div>

    <script>
        document.querySelectorAll('input[name="cmntxpt_captcha_type"]').forEach((radio) => {
            radio.addEventListener('change', function() {
                document.getElementById('recaptcha-v2-settings').style.display = this.value === 'v2' ? 'block' : 'none';
                document.getElementById('recaptcha-v3-settings').style.display = this.value === 'v3' ? 'block' : 'none';
                document.getElementById('hcaptcha-settings').style.display = this.value === 'hcaptcha' ? 'block' : 'none';
            });
        });
    </script>
    <?php
}

if(cmntxpt_is_comment_captcha_enabled()){
	// In your theme or plugin, when rendering the comment form:
	function cmntxpt_add_comment_nonce_field() {
		wp_nonce_field('cmntxpt_comment_nonce_action', 'cmntxpt_comment_nonce');
	}
	add_action('comment_form', 'cmntxpt_add_comment_nonce_field');
	
	// Verify CAPTCHA and Nonce on comment submission
	function cmntxpt_verify_captcha_on_comment($commentdata) {
		if ( ! isset($_POST['cmntxpt_comment_nonce']) ) {
			wp_die(esc_html__('Nonce is missing. Please try again.', 'commentxpert'));
		}

		// Unsplash and sanitize the nonce before verifying
		$nonce = sanitize_text_field(wp_unslash($_POST['cmntxpt_comment_nonce']));

		if ( ! wp_verify_nonce($nonce, 'cmntxpt_comment_nonce_action') ) {
			wp_die(esc_html__('Nonce verification failed. Please try again.', 'commentxpert'));
		}

		$captcha_type = get_option('cmntxpt_captcha_type', 'v2');
		$threshold = get_option('cmntxpt_recaptcha_v3_threshold', 0.5);

		if (!$captcha_type || (!isset($_POST['g-recaptcha-response']) && !isset($_POST['h-captcha-response']))) {
			wp_die(esc_html__('Please complete the CAPTCHA to submit your comment.', 'commentxpert'));
		}

		// Unsplash and sanitize CAPTCHA response
		$response_key = sanitize_text_field(wp_unslash($_POST['g-recaptcha-response'] ?? $_POST['h-captcha-response']));
		$secret_key = get_option('cmntxpt_' . ($captcha_type === 'hcaptcha' ? 'hcaptcha_secret_key' : 'recaptcha_' . ($captcha_type === 'v3' ? 'v3_' : '') . 'secret_key'));

		$verify_url = $captcha_type === 'hcaptcha' 
			? 'https://hcaptcha.com/siteverify' 
			: 'https://www.google.com/recaptcha/api/siteverify';

		// Sanitize and validate the user's IP address
		$remote_ip = isset($_SERVER['REMOTE_ADDR']) ? sanitize_text_field(wp_unslash($_SERVER['REMOTE_ADDR'])) : '';

		if (empty($remote_ip) || !filter_var($remote_ip, FILTER_VALIDATE_IP)) {
			$remote_ip = '';
		}

		$response = wp_remote_post($verify_url, array(
			'body' => array(
				'secret' => $secret_key,
				'response' => $response_key,
				'remoteip' => $remote_ip,
			),
		));

		$response_body = wp_remote_retrieve_body($response);
		$result = json_decode($response_body, true);

		if (!$result['success'] || ($captcha_type === 'v3' && isset($result['score']) && $result['score'] < $threshold)) {
			wp_die(esc_html__('CAPTCHA verification failed. Please try again.', 'commentxpert'));
		}

		return $commentdata;
	}
	add_filter('preprocess_comment', 'cmntxpt_verify_captcha_on_comment');
	
	// Initialize the selected CAPTCHA in the comment form
	function cmntxpt_add_captcha_to_comment_form() {
		$captcha_type = get_option('cmntxpt_captcha_type', 'v2');
		
		$site_key = get_option('cmntxpt_' . ($captcha_type === 'hcaptcha' ? 'hcaptcha_site_key' : 'recaptcha_' . ($captcha_type === 'v3' ? 'v3_' : '') . 'site_key'));
	
		if (!$site_key) return;

		if ($captcha_type === 'v2') {
            echo '<div class="g-recaptcha" data-sitekey="' . esc_attr($site_key) . '"></div>';
            wp_enqueue_script('google-recaptcha-v2', 'https://www.google.com/recaptcha/api.js', array(), microtime(), true);
        } elseif ($captcha_type === 'v3') {
            echo '<input type="hidden" id="g-recaptcha-response" name="g-recaptcha-response">';
            wp_enqueue_script('google-recaptcha-v3', 'https://www.google.com/recaptcha/api.js?render=' . esc_attr($site_key), array(), microtime(), true);

            // inline script to handle the token
            wp_add_inline_script('google-recaptcha-v3', "
                grecaptcha.ready(function() {
                    grecaptcha.execute('" . esc_js($site_key) . "', {action: 'submit'}).then(function(token) {
                        document.getElementById('g-recaptcha-response').value = token;
                    });
                });
            ");
        } elseif ($captcha_type === 'hcaptcha') {
            echo '<div class="h-captcha" data-sitekey="' . esc_attr($site_key) . '"></div>';
            wp_enqueue_script('hcaptcha', 'https://js.hcaptcha.com/1/api.js', array(), microtime(), true);
        }
	}
	add_action('comment_form_after_fields', 'cmntxpt_add_captcha_to_comment_form');
	
}

if(cmntxpt_is_rich_text_editor_enabled()){
	// Enqueue TinyMCE for the default WordPress comment form
	function cmntxpt_enqueue_tinymce() {
		if (is_singular() && comments_open()) {
			wp_enqueue_script('wp-tinymce');
			add_action('wp_footer', 'cmntxpt_initialize_tinymce');
		}
	}
	add_action('wp_enqueue_scripts', 'cmntxpt_enqueue_tinymce');

	// Initialize TinyMCE with security restrictions
	function cmntxpt_initialize_tinymce() { ?>
		<script>
			document.addEventListener('DOMContentLoaded', function () {
				const commentField = document.querySelector('#comment');
				if (commentField) {
					commentField.addEventListener('focus', function () {
						if (!commentField.classList.contains('tinymce-loaded')) {
							commentField.classList.add('tinymce-loaded');

							tinymce.init({
								selector: '#comment',
								menubar: false,
								toolbar: 'bold italic underline | bullist numlist | link',
								plugins: 'lists link',
								valid_elements: 'a[href|title|rel],strong,em,u,ul,ol,li,p,br',
								setup: function(editor) {
									editor.on('change', function () {
										tinymce.triggerSave();
									});
								}
							});
						}
					});
				}
			});
		</script>
	<?php }

	// Sanitize the comment input to allow only safe HTML elements
	function cmntxpt_sanitize_comment_input($commentdata) {
		$allowed_tags = array(
			'a'      => array('href' => array(), 'title' => array(), 'rel' => array()),
			'strong' => array(),
			'em'     => array(),
			'u'      => array(),
			'ul'     => array(),
			'ol'     => array(),
			'li'     => array(),
			'p'      => array(),
			'br'     => array(),
		);

		$commentdata['comment_content'] = wp_kses($commentdata['comment_content'], $allowed_tags);
		return $commentdata;
	}
	add_filter('preprocess_comment', 'cmntxpt_sanitize_comment_input');

	// Escape comment output to prevent XSS when displaying comments
	function cmntxpt_escape_comment_output($comment_text) {
		return wp_kses_post($comment_text);
	}
	add_filter('comment_text', 'cmntxpt_escape_comment_output', 10, 2);	
}